@@ -44,9 +44,9 @@
 #include <e-util/e-plugin.h>
 
 #include <libebook/e-contact.h>
-#include <libebook/e-book.h>
+#include <libebook/e-book-client.h>
 
-#include <libecal/e-cal.h>
+#include <libecal/e-cal-client.h>
 #include <libecal/e-cal-component.h>
 
 #include <libedataserver/e-data-server-util.h>
@@ -59,9 +59,6 @@
 #include <mail/mail-tools.h>
 #include <mail/em-utils.h>
 #include <shell/e-shell.h>
-#include <shell/e-shell-window.h>
-#include <shell/e-shell-view.h>
-#include <shell/e-shell-sidebar.h>
 
 #include <libpst/libpst.h>
 #include <libpst/timeconv.h>
@@ -79,7 +76,7 @@ gint pst_init (pst_file *pst, gchar *filename);
 gchar *get_pst_rootname (pst_file *pst, gchar *filename);
 static void pst_error_msg (const gchar *fmt, ...);
 static void pst_import_folders (PstImporter *m, pst_desc_tree *topitem);
-static void pst_process_item (PstImporter *m, pst_desc_tree *d_ptr, gchar **previouss_folder);
+static void pst_process_item (PstImporter *m, pst_desc_tree *d_ptr);
 static void pst_process_folder (PstImporter *m, pst_item *item);
 static void pst_process_email (PstImporter *m, pst_item *item);
 static void pst_process_contact (PstImporter *m, pst_item *item);
@@ -122,15 +119,16 @@ struct _PstImporter {
 	pst_file pst;
 
 	CamelFolder *folder;
+	gchar *parent_uri;
 	gchar *folder_name;
 	gchar *folder_uri;
 	gint folder_count;
 	gint current_item;
 
-	EBook *addressbook;
-	ECal *calendar;
-	ECal *tasks;
-	ECal *journal;
+	EBookClient *addressbook;
+	ECalClient *calendar;
+	ECalClient *tasks;
+	ECalClient *journal;
 };
 
 gboolean
@@ -146,7 +144,7 @@ org_credativ_evolution_readpst_supported (EPlugin *epl, EImportTarget *target)
 		return FALSE;
 	}
 
-	s = (EImportTargetURI *)target;
+	s = (EImportTargetURI *) target;
 
 	if (s->uri_src == NULL) {
 		return TRUE;
@@ -215,7 +213,6 @@ get_suggested_foldername (EImportTargetURI *target)
 	EShell *shell;
 	EShellBackend *shell_backend;
 	EMailSession *session;
-	GtkWindow *window;
 	const gchar *inbox;
 	gchar *delim, *filename;
 	gchar *rootname = NULL;
@@ -229,58 +226,18 @@ get_suggested_foldername (EImportTargetURI *target)
 	shell_backend = e_shell_get_backend_by_name (shell, "mail");
 	session = e_mail_backend_get_session (E_MAIL_BACKEND (shell_backend));
 
-	foldername = NULL;
+	/* Suggest a folder that is in the same mail storage as the users' inbox,
+	   with a name derived from the .PST file */
+	inbox = e_mail_local_get_folder_uri (E_MAIL_LOCAL_FOLDER_INBOX);
 
-	/* preselect the folder selected in a mail view */
-	window = e_shell_get_active_window (shell);
-	if (E_IS_SHELL_WINDOW (window)) {
-		EShellWindow *shell_window;
-		const gchar *view;
-
-		shell_window = E_SHELL_WINDOW (window);
-		view = e_shell_window_get_active_view (shell_window);
-
-		if (view && g_str_equal (view, "mail")) {
-			EShellView *shell_view;
-			EShellSidebar *shell_sidebar;
-			EMFolderTree *folder_tree = NULL;
-			gchar *selected_uri;
-
-			shell_view = e_shell_window_get_shell_view (
-				shell_window, view);
-
-			shell_sidebar =
-				e_shell_view_get_shell_sidebar (shell_view);
-
-			g_object_get (
-				shell_sidebar, "folder-tree",
-				&folder_tree, NULL);
-
-			selected_uri = em_folder_tree_get_selected_uri (folder_tree);
-
-			g_object_unref (folder_tree);
-
-			if (selected_uri && *selected_uri)
-				foldername = g_string_new (selected_uri);
-
-			g_free (selected_uri);
-		}
+	delim = g_strrstr (inbox, "#");
+	if (delim != NULL) {
+		foldername = g_string_new_len (inbox, delim-inbox);
+	} else {
+		foldername = g_string_new (inbox);
 	}
 
-	if (!foldername) {
-		/* Suggest a folder that is in the same mail storage as the users' inbox,
-		   with a name derived from the .PST file */
-		inbox = e_mail_local_get_folder_uri (E_MAIL_LOCAL_FOLDER_INBOX);
-
-		delim = g_strrstr (inbox, "#");
-		if (delim != NULL) {
-			foldername = g_string_new_len (inbox, delim-inbox);
-		} else {
-			foldername = g_string_new (inbox);
-		}
-	}
-
-	g_string_append_c (foldername, '/');
+	g_string_append_c (foldername, '#');
 
 	filename = g_filename_from_uri (target->uri_src, NULL, NULL);
 
@@ -335,7 +292,6 @@ org_credativ_evolution_readpst_getwidget (EImport *ei, EImportTarget *target, EI
 {
 	EShell *shell;
 	EShellBackend *shell_backend;
-	EMailSession *session;
 	GtkWidget *hbox, *framebox, *w;
 	gchar *foldername;
 
@@ -356,9 +312,11 @@ org_credativ_evolution_readpst_getwidget (EImport *ei, EImportTarget *target, EI
 
 	shell = e_shell_get_default ();
 	shell_backend = e_shell_get_backend_by_name (shell, "mail");
-	session = e_mail_backend_get_session (E_MAIL_BACKEND (shell_backend));
 
-	w = em_folder_selection_button_new (session, _("Select folder"), _("Select folder to import into"));
+	w = em_folder_selection_button_new (
+		E_MAIL_BACKEND (shell_backend),
+		_("Select folder"),
+		_("Select folder to import into"));
 	foldername = get_suggested_foldername ((EImportTargetURI *) target);
 	((EImportTargetURI *) target)->uri_dest = g_strdup (foldername);
 	em_folder_selection_button_set_selection ((EMFolderSelectionButton *) w, foldername);
@@ -381,19 +339,19 @@ org_credativ_evolution_readpst_getwidget (EImport *ei, EImportTarget *target, EI
 	w = gtk_check_button_new_with_mnemonic (_("A_ppointments"));
 	gtk_toggle_button_set_active ((GtkToggleButton *) w, FALSE);
 	g_signal_connect (w, "toggled", G_CALLBACK (checkbox_appt_toggle_cb), target);
-	gtk_box_pack_start ((GtkBox *)framebox, w, FALSE, FALSE, 0);
+	gtk_box_pack_start ((GtkBox *) framebox, w, FALSE, FALSE, 0);
 
 	/* Tasks */
 	w = gtk_check_button_new_with_mnemonic (_("_Tasks"));
-	gtk_toggle_button_set_active ((GtkToggleButton *)w, FALSE);
+	gtk_toggle_button_set_active ((GtkToggleButton *) w, FALSE);
 	g_signal_connect (w, "toggled", G_CALLBACK (checkbox_task_toggle_cb), target);
-	gtk_box_pack_start ((GtkBox *)framebox, w, FALSE, FALSE, 0);
+	gtk_box_pack_start ((GtkBox *) framebox, w, FALSE, FALSE, 0);
 
 	/* Journal */
 	w = gtk_check_button_new_with_mnemonic (_("_Journal entries"));
-	gtk_toggle_button_set_active ((GtkToggleButton *)w, FALSE);
+	gtk_toggle_button_set_active ((GtkToggleButton *) w, FALSE);
 	g_signal_connect (w, "toggled", G_CALLBACK (checkbox_journal_toggle_cb), target);
-	gtk_box_pack_start ((GtkBox *)framebox, w, FALSE, FALSE, 0);
+	gtk_box_pack_start ((GtkBox *) framebox, w, FALSE, FALSE, 0);
 
 	gtk_widget_show_all (framebox);
 
@@ -408,29 +366,41 @@ pst_import_describe (PstImporter *m, gint complete)
 	return g_strdup (_("Importing Outlook data"));
 }
 
-static ECal*
-open_ecal (ECalSourceType type, const gchar *name)
+static ECalClient *
+open_ecal (ECalClientSourceType type, const gchar *name)
 {
 	/* Hack - grab the first calendar we can find
 		TODO - add a selection mechanism in get_widget */
 	ESource *primary;
 	ESourceList *source_list;
-	ECal *cal;
+	ECalClient *cal;
+	GError *error = NULL;
 
-	if ((e_cal_get_sources (&source_list, type, NULL)) == 0) {
-		g_warning ("Could not get any sources of type %s.", name);
+	if ((e_cal_client_get_sources (&source_list, type, &error)) == 0) {
+		g_debug ("%s: Could not get any sources of type %s: %s", G_STRFUNC, name, error ? error->message : "Unknown error");
+		if (error)
+			g_error_free (error);
 		return NULL;
 	}
 
 	primary = e_source_list_peek_source_any (source_list);
 
-	if ((cal = e_cal_new (primary, type)) == NULL) {
-		g_warning ("Could not create %s.", name);
+	if ((cal = e_cal_client_new (primary, type, &error)) == NULL) {
+		g_debug ("%s: Could not create %s: %s", G_STRFUNC, name, error ? error->message : "Unknown error");
+		if (error)
+			g_error_free (error);
 		g_object_unref (source_list);
 		return NULL;
 	}
 
-	e_cal_open (cal, TRUE, NULL);
+	if (!e_client_open_sync (E_CLIENT (cal), TRUE, NULL, &error)) {
+		g_debug ("%s: Failed to open %s: %s", G_STRFUNC, name, error ? error->message : "Unknown error");
+		if (error)
+			g_error_free (error);
+		g_object_unref (cal);
+		cal = NULL;
+	}
+
 	g_object_unref (primary);
 	g_object_unref (source_list);
 
@@ -447,32 +417,39 @@ pst_import_import (PstImporter *m,
 		   TODO - add a selection mechanism in get_widget */
 		ESource *primary;
 		ESourceList *source_list;
+		GError *error = NULL;
 
-		if (e_book_get_addressbooks (&source_list, NULL)) {
+		if (e_book_client_get_sources (&source_list, &error)) {
 			primary = e_source_list_peek_source_any (source_list);
 
-			if ((m->addressbook = e_book_new (primary,NULL))) {
-				e_book_open (m->addressbook, TRUE, NULL);
+			if ((m->addressbook = e_book_client_new (primary, &error))) {
+				if (!e_client_open_sync (E_CLIENT (m->addressbook), TRUE, NULL, &error)) {
+					g_debug ("%s: Failed to open book client: %s", G_STRFUNC, error ? error->message : "Unknown error");
+				}
+
 				g_object_unref (primary);
 				g_object_unref (source_list);
 			} else {
-				g_warning ("Could not create EBook.");
+				g_debug ("%s: Could not create book client: %s", G_STRFUNC, error ? error->message : "Unknown error");
 			}
 		} else {
-			g_warning ("Could not get address books.");
+			g_debug ("%s: Could not get address books: %s", G_STRFUNC, error ? error->message : "Unknown error");
 		}
+
+		if (error)
+			g_error_free (error);
 	}
 
 	if (GPOINTER_TO_INT (g_datalist_get_data (&m->target->data, "pst-do-appt"))) {
-		m->calendar = open_ecal (E_CAL_SOURCE_TYPE_EVENT, "calendar");
+		m->calendar = open_ecal (E_CAL_CLIENT_SOURCE_TYPE_EVENTS, "calendar");
 	}
 
 	if (GPOINTER_TO_INT (g_datalist_get_data (&m->target->data, "pst-do-task"))) {
-		m->tasks = open_ecal (E_CAL_SOURCE_TYPE_TODO, "task list");
+		m->tasks = open_ecal (E_CAL_CLIENT_SOURCE_TYPE_TASKS, "task list");
 	}
 
 	if (GPOINTER_TO_INT (g_datalist_get_data (&m->target->data, "pst-do-journal"))) {
-		m->journal = open_ecal (E_CAL_SOURCE_TYPE_JOURNAL, "journal");
+		m->journal = open_ecal (E_CAL_CLIENT_SOURCE_TYPE_MEMOS, "journal");
 	}
 
 	pst_import_file (m);
@@ -517,14 +494,14 @@ pst_import_file (PstImporter *m)
 	shell_backend = e_shell_get_backend_by_name (shell, "mail");
 	session = e_mail_backend_get_session (E_MAIL_BACKEND (shell_backend));
 
-	filename = g_filename_from_uri (((EImportTargetURI *)m->target)->uri_src, NULL, NULL);
-	m->folder_uri = g_strdup (((EImportTargetURI *)m->target)->uri_dest); /* Destination folder, was set in our widget */
+	filename = g_filename_from_uri (((EImportTargetURI *) m->target)->uri_src, NULL, NULL);
+	m->parent_uri = g_strdup (((EImportTargetURI *) m->target)->uri_dest); /* Destination folder, was set in our widget */
 
 	camel_operation_push_message (NULL, _("Importing '%s'"), filename);
 
 	if (GPOINTER_TO_INT (g_datalist_get_data (&m->target->data, "pst-do-mail"))) {
 		e_mail_session_uri_to_folder_sync (
-			session, m->folder_uri, CAMEL_STORE_FOLDER_CREATE,
+			session, m->parent_uri, CAMEL_STORE_FOLDER_CREATE,
 			cancellable, &m->base.error);
 	}
 
@@ -566,74 +543,49 @@ pst_import_file (PstImporter *m)
 static void
 pst_import_folders (PstImporter *m, pst_desc_tree *topitem)
 {
-	GHashTable *node_to_folderuri; /* pointers of hierarchy nodes, to them associated folder uris */
 	pst_desc_tree *d_ptr;
+	gchar *seperator;
 
-	node_to_folderuri = g_hash_table_new_full (g_direct_hash, g_direct_equal, NULL, g_free);
 	d_ptr = topitem->child;
 
-	if (topitem)
-		g_hash_table_insert (node_to_folderuri, topitem, g_strdup (m->folder_uri));
-		
 	/* Walk through folder tree */
 	while (d_ptr != NULL && (camel_operation_cancel_check (NULL) == FALSE)) {
-		gchar *previous_folder = NULL;
 
-		pst_process_item (m, d_ptr, &previous_folder);
+		pst_process_item (m, d_ptr);
 
 		if (d_ptr->child != NULL) {
-			if (m->folder) {
-				g_object_unref (m->folder);
-				m->folder = NULL;
-			}
-
-			g_return_if_fail (m->folder_uri != NULL);
-			g_hash_table_insert (node_to_folderuri, d_ptr, g_strdup (m->folder_uri));
-
+			g_free (m->parent_uri);
+			m->parent_uri = g_strdup (m->folder_uri);
 			d_ptr = d_ptr->child;
 		} else if (d_ptr->next != NULL) {
-			/* for cases where there is an empty folder node, with no subnodes */
-			if (previous_folder) {
-				g_free (m->folder_uri);
-				m->folder_uri = previous_folder;
-				previous_folder = NULL;
-			}
-
 			d_ptr = d_ptr->next;
 		} else {
 			while (d_ptr != topitem && d_ptr->next == NULL) {
-				if (m->folder) {
-					g_object_unref (m->folder);
-					m->folder = NULL;
+				if (m->folder_uri) {
+					g_free (m->folder_uri);
 				}
 
-				g_free (m->folder_uri);
-				m->folder_uri = NULL;
+				m->folder_uri = g_strdup (m->parent_uri);
+				seperator = g_strrstr (m->parent_uri, "/");
+
+				if (seperator != NULL) {
+					*seperator = '\0'; /* Truncate uri */
+				}
 
 				d_ptr = d_ptr->parent;
 
-				if (d_ptr && d_ptr != topitem) {
-					m->folder_uri = g_strdup (g_hash_table_lookup (node_to_folderuri, d_ptr->parent));
-					g_return_if_fail (m->folder_uri != NULL);
-				}
 			}
-
 			if (d_ptr == topitem) {
-				g_free (previous_folder);
-				break;
+				return;
 			}
 
 			d_ptr = d_ptr->next;
 		}
-
-		g_free (previous_folder);
 	}
-
-	g_hash_table_destroy (node_to_folderuri);
 }
 
 static void
-pst_process_item (PstImporter *m, pst_desc_tree *d_ptr, gchar **previous_folder)
+pst_process_item (PstImporter *m, pst_desc_tree *d_ptr)
 {
 	pst_item *item = NULL;
 
@@ -652,8 +604,6 @@ pst_process_item (PstImporter *m, pst_desc_tree *d_ptr, gchar **previous_folder)
 	}
 
 	if (item->folder != NULL) {
-		if (previous_folder)
-			*previous_folder = g_strdup (m->folder_uri);
 		pst_process_folder (m, item);
 		camel_operation_push_message (
 			NULL, _("Importing '%s'"), item->file_as.str);
@@ -664,29 +614,36 @@ pst_process_item (PstImporter *m, pst_desc_tree *d_ptr, gchar **previous_folder)
 			camel_operation_progress (NULL, 100);
 		}
 
-		switch (item->type) {
-		case PST_TYPE_CONTACT:
-			if (item->contact && m->addressbook && GPOINTER_TO_INT (g_datalist_get_data (&m->target->data, "pst-do-addr")))
-				pst_process_contact (m, item);
-			break;
-		case PST_TYPE_APPOINTMENT:
-			if (item->appointment && m->calendar && GPOINTER_TO_INT (g_datalist_get_data (&m->target->data, "pst-do-appt")))
-				pst_process_appointment (m, item);
-			break;
-		case PST_TYPE_TASK:
-			if (item->appointment && m->tasks && GPOINTER_TO_INT (g_datalist_get_data (&m->target->data, "pst-do-task")))
-				pst_process_task (m, item);
-			break;
-		case PST_TYPE_JOURNAL:
-			if (item->appointment && m->journal && GPOINTER_TO_INT (g_datalist_get_data (&m->target->data, "pst-do-journal")))
-				pst_process_journal (m, item);
-			break;
-		case PST_TYPE_NOTE:
-		case PST_TYPE_SCHEDULE:
-		case PST_TYPE_REPORT:
-			if (item->email && GPOINTER_TO_INT (g_datalist_get_data (&m->target->data, "pst-do-mail")))
+		if (item->email != NULL &&
+			(item->type == PST_TYPE_NOTE || item->type == PST_TYPE_REPORT)) {
+
+			if (GPOINTER_TO_INT (g_datalist_get_data (&m->target->data, "pst-do-mail")))
 				pst_process_email (m, item);
-			break;
+
+		} else if (item->contact && item->type == PST_TYPE_CONTACT) {
+
+			if (m->addressbook && GPOINTER_TO_INT (g_datalist_get_data (&m->target->data, "pst-do-addr"))) {
+				pst_process_contact (m, item);
+			}
+
+		} else if (item->type == PST_TYPE_APPOINTMENT && item->appointment) {
+
+			if (m->calendar && GPOINTER_TO_INT (g_datalist_get_data (&m->target->data, "pst-do-appt"))) {
+				pst_process_appointment (m, item);
+			}
+
+		} else if (item->type == PST_TYPE_TASK && item->appointment) {
+
+			if (m->tasks && GPOINTER_TO_INT (g_datalist_get_data (&m->target->data, "pst-do-task"))) {
+				pst_process_task (m, item);
+			}
+
+		} else if (item->type == PST_TYPE_JOURNAL && item->appointment) {
+
+			if (m->journal && GPOINTER_TO_INT (g_datalist_get_data (&m->target->data, "pst-do-journal"))) {
+				pst_process_journal (m, item);
+			}
+
 		}
 
 		m->current_item++;
@@ -756,6 +713,7 @@ pst_process_folder (PstImporter *m, pst_item *item)
 {
 	gchar *uri;
 	g_free (m->folder_name);
+	g_free (m->folder_uri);
 
 	if (item->file_as.str != NULL) {
 		m->folder_name = foldername_to_utf8 (item->file_as.str);
@@ -764,8 +722,7 @@ pst_process_folder (PstImporter *m, pst_item *item)
 		m->folder_name = g_strdup ("unknown_name");
 	}
 
-	uri = g_strjoin ("/", m->folder_uri, m->folder_name, NULL);
-	g_free (m->folder_uri);
+	uri = g_strjoin ("/", m->parent_uri, m->folder_name, NULL);
 	m->folder_uri = uri;
 
 	if (m->folder) {
@@ -804,10 +761,10 @@ pst_create_folder (PstImporter *m)
 	shell_backend = e_shell_get_backend_by_name (shell, "mail");
 	session = e_mail_backend_get_session (E_MAIL_BACKEND (shell_backend));
 
-	parent = ((EImportTargetURI *)m->target)->uri_dest;
+	parent = ((EImportTargetURI *) m->target)->uri_dest;
 	dest = g_strdup (m->folder_uri);
 
-	g_return_if_fail (g_str_has_prefix (dest, parent));
+	g_assert (g_str_has_prefix (dest, parent));
 
 	if (m->folder) {
 		g_object_unref (m->folder);
@@ -954,7 +911,7 @@ pst_process_email (PstImporter *m, pst_item *item)
 		/*g_message ("  Email headers... %s...", item->email->header);*/
 
 		stream = camel_stream_mem_new_with_buffer (item->email->header.str, strlen (item->email->header.str));
-		if (!camel_data_wrapper_construct_from_stream_sync ((CamelDataWrapper *)msg, stream, NULL, NULL))
+		if (!camel_data_wrapper_construct_from_stream_sync ((CamelDataWrapper *) msg, stream, NULL, NULL))
 			g_warning ("Error reading headers, skipped");
 
 	} else {
@@ -1092,7 +1049,7 @@ contact_set_address (EContact *contact, EContactField id, gchar *address, gchar
 		if (po_box) {
 			eaddress->po = g_strdup (po_box);
 		}
-		/* eaddress->ext = */
+		//eaddress->ext =
 
 		if (street) {
 			eaddress->street = g_strdup (street);
@@ -1144,6 +1101,8 @@ pst_process_contact (PstImporter *m, pst_item *item)
 	pst_item_contact *c;
 	EContact *ec;
 	GString *notes;
+	gchar *uid = NULL;
+	GError *error = NULL;
 
 	c = item->contact;
 	notes = g_string_sized_new (2048);
@@ -1280,9 +1239,16 @@ pst_process_contact (PstImporter *m, pst_item *item)
 	contact_set_string (ec, E_CONTACT_NOTE, notes->str);
 	g_string_free (notes, TRUE);
 
-	e_book_add_contact (m->addressbook, ec, NULL);
-	g_object_unref (ec);
+	if (!e_book_client_add_contact_sync (m->addressbook, ec, &uid, NULL, &error))
+		uid = NULL;
 
+	g_object_unref (ec);
+	g_free (uid);
+
+	if (error) {
+		g_debug ("%s: Failed to add contact: %s", G_STRFUNC, error->message);
+		g_error_free (error);
+	}
 }
 
 /**
@@ -1308,7 +1274,7 @@ get_ical_date (FILETIME *date, gboolean is_date)
 }
 
 static void
-set_cal_attachments (ECal *cal, ECalComponent *ec, PstImporter *m, pst_item_attach *attach)
+set_cal_attachments (ECalClient *cal, ECalComponent *ec, PstImporter *m, pst_item_attach *attach)
 {
 	GSList *list = NULL;
 	const gchar *uid;
@@ -1319,7 +1285,7 @@ set_cal_attachments (ECal *cal, ECalComponent *ec, PstImporter *m, pst_item_atta
 	}
 
 	e_cal_component_get_uid (ec, &uid);
-	store_dir = g_filename_from_uri (e_cal_get_local_attachment_store (cal), NULL, NULL);
+	store_dir = g_filename_from_uri (e_cal_client_get_local_attachment_store (cal), NULL, NULL);
 
 	while (attach != NULL) {
 		const gchar * orig_filename;
@@ -1476,7 +1442,7 @@ fill_calcomponent (PstImporter *m, pst_item *item, ECalComponent *ec, const gcha
 			e_cal_component_set_status (ec, ICAL_STATUS_TENTATIVE);
 			break;
 		case PST_FREEBUSY_FREE:
-			/* mark as transparent and as confirmed */
+			// mark as transparent and as confirmed
 			e_cal_component_set_transparency (ec, E_CAL_COMPONENT_TRANSP_TRANSPARENT);
 		case PST_FREEBUSY_BUSY:
 		case PST_FREEBUSY_OUT_OF_OFFICE:
@@ -1564,9 +1530,10 @@ fill_calcomponent (PstImporter *m, pst_item *item, ECalComponent *ec, const gcha
 }
 
 static void
-pst_process_component (PstImporter *m, pst_item *item, const gchar *comp_type, ECal *cal)
+pst_process_component (PstImporter *m, pst_item *item, const gchar *comp_type, ECalClient *cal)
 {
 	ECalComponent *ec;
+	gchar *uid = NULL;
 	GError *error = NULL;
 
 	g_return_if_fail (item->appointment != NULL);
@@ -1577,12 +1544,13 @@ pst_process_component (PstImporter *m, pst_item *item, const gchar *comp_type, E
 	fill_calcomponent (m, item, ec, comp_type);
 	set_cal_attachments (cal, ec, m, item->attach);
 
-	if (!e_cal_create_object (cal, e_cal_component_get_icalcomponent (ec), NULL, &error)) {
+	if (!e_cal_client_create_object_sync (cal, e_cal_component_get_icalcomponent (ec), &uid, NULL, &error)) {
+		uid = NULL;
 		g_warning ("Creation of %s failed: %s", comp_type, error ? error->message : "Unknown error");
 	}
 
 	g_object_unref (ec);
-
+	g_free (uid);
 	if (error)
 		g_error_free (error);
 }
@@ -1619,13 +1587,13 @@ pst_error_msg (const gchar *fmt, ...)
 static void
 pst_import_imported (PstImporter *m)
 {
-	e_import_complete (m->target->import, (EImportTarget *)m->target);
+	e_import_complete (m->target->import, (EImportTarget *) m->target);
 }
 
 static void
 pst_import_free (PstImporter *m)
 {
-	/* pst_close (&m->pst); */
+//	pst_close (&m->pst);
 	g_object_unref (m->cancellable);
 
 	g_free (m->status_what);
@@ -1636,6 +1604,7 @@ pst_import_free (PstImporter *m)
 
 	g_free (m->folder_name);
 	g_free (m->folder_uri);
+	g_free (m->parent_uri);
 
 	g_object_unref (m->import);
 }
@@ -1662,7 +1631,7 @@ pst_status_timeout (gpointer data)
 		pc = importer->status_pc;
 		g_mutex_unlock (importer->status_lock);
 
-		e_import_status (importer->target->import, (EImportTarget *)importer->target, what, pc);
+		e_import_status (importer->target->import, (EImportTarget *) importer->target, what, pc);
 	}
 
 	return TRUE;
@@ -1692,6 +1661,7 @@ pst_import (EImport *ei, EImportTarget *target)
 	g_object_ref (m->import);
 	m->target = target;
 
+	m->parent_uri = NULL;
 	m->folder_name = NULL;
 	m->folder_uri = NULL;
 
